import tensorflow as tf
import matplotlib.pyplot as plt
import numpy as np
import re


# You can skip this function; it is simply the training from above plus some
# interactive visualization
def perform_training_with_visualization(steps, batch_size, dataset, model):
    dataset = dataset(batch_size)
    next_image, next_label = dataset.make_one_shot_iterator().get_next()
    model_outputs = model(next_image, next_label)
    loss = model_outputs["loss"]
    opt_step = model_outputs["opt_step"]
    with tf.Session() as sess:
        sess.run(tf.global_variables_initializer())

        # setup plot
        fig = plt.figure()
        ax = fig.add_subplot(111)
        plt.ion()
        fig.show()
        fig.canvas.draw()
        loss_points = []
        time_steps = np.arange((steps / 5) + 1) * 5

        for i in xrange(steps):
            _, loss_value = sess.run((opt_step, loss))

            # update plot
            if i % 5 == 0:
                ax.clear()
                loss_points.append(loss_value)
                ax.plot(time_steps[:i / 5 + 1], loss_points)
                ax.set_xlim([0, steps])
                ax.set_ylim([0, 2.5])
                fig.canvas.draw()
        print("Final Loss: %f" % loss_value)


# This function filters the model's variables by some regular expression
def get_variables_for_saver(include=None, exclude=None):
    variables = tf.get_collection(tf.GraphKeys.GLOBAL_VARIABLES)

    if include is []:
        print("The include list for the initilization is the empty list. "
              "Is that intentional? No variables are saved/loaded!")
    elif include is not None:
        regexs = map(re.compile, include)
        variables = filter(lambda v: any(map(lambda r: r.match(v.name), regexs)), variables)
    if exclude is not None:

        for e in exclude:
            regex = re.compile(e)
            variables = filter(lambda v: not regex.match(v.name), variables)
    return list(variables)
